% Autores: Herbert Martins Gomes e Ewerton Grotti.
% Fonte: cl, B.6, pp.199. DTU Emil Brandt Krgaard thesis 
% Multidimensional Collocation Method SCM.

%% Programa para anlise de intervalo
clc;clear all;close all;
global chamadas
chamadas=0;
imin=[0.9];
imax=[1.1];
%% Parametros do PCE
level=3; %Numero de termos da expanso, depende da complexidade da funo.  
gdlan=[1];%[91]     %Graus de liberdade da anlise numrica que se deseja fazer anlise de intervalo
nptun=30;           %Nmero de pontos no tempo para fazer a anlise de incerteza ( menor que o numero de pontos da anlise no tempo do sistema)
nptunn=1000;        %Nmero de intervalos de tempo a ser suavizado ( nptunn > nptun)
notva=length(gdlan);%Nmero de variveis incertas de sada
T=0.3;%5.0;         %Tempo total da anlise
dt=1.0e-3;          %incremento de tempo na anlise dinmica
%% para inserir mais variaveis incertas, simplesmente acrescentar os elementos com seus limites em ub e lb
%esse programa s serve para variaveis de entrada com distribuio
%uniforme. para alterar isso, teria que mudar as regras de quadratura.
d=length(imin);     %Numero de variveis
lb=imin;            %Valores Minimos do intervalo para variveis de entrada
ub=imax;            %Valores Mximos do intervalo para variveis de entrada
for i=1:length(ub)
    param(:,i)=[lb(i),ub(i)];
end
% Computing the number of grid points
Pn=sparse_grid_gl_size(d,level);
% Computing sparse grid
[gw, gp]=sparse_grid_gl(d,level,Pn);
gp=gp';
%Scaling the disturbances.
for i=1:d
    XX(:,i)=((gp(:,i)+1)/2).*(ub(i)-lb(i))+lb(i);
end
% Scaling the weights.
ZW=gw/(2^d);    %Transformao misteriosa
AN=length(gw) ; %Numero de pontos de integrao
% Solving deterministic system.
iptun=nptun;
% parfor i=1:AN
for i=1:AN
    [~, ~, ~, signals(i,:,:)]=fcn(XX(i,:),iptun,nptun,gdlan(notva),T,dt,0);%x,v,a: (pontos_de_integrao_do_PCE,gld,tempo), XX: (pontos_de_integrao_do_PCE,gdl); as ultimas 3 variveis: iptun,nptun,idlan, s completam a funo para que fique compativel com a anlise de intervalo.
end
fcn_calls_PCE=chamadas;
chamadas=0;
% [~, ~, ~, ~]=fcn((lb+ub)/2,iptun,nptun,gdlan(notva),T,dt,1);%plota sem incertezas
%% organizando os dados. Talvez seja necessrio adaptar dependendo do OUTPUT da FCN.  
n_output=size(signals,2)/3;%supondo que sempre sairemos com 3 tipos de sinais: Deslocamento, Velocidade e Accelerao, nessa ordem. 
%Pode haver mais de um sinal do mesmo tipo. Este loop serve para ordenar os sinais juntos, por exemplo: signals=[Posio; Posio; Vel.; Vel.; Acc. Acc.]. (programado para esta ordem).
x(:,1:n_output,:)=signals(:,1:n_output,:);%dimenses de signals:(Pn,3*n_output,T/dt)
v(:,1:n_output,:)=signals(:,n_output+1:n_output*2,:);
a(:,1:n_output,:)=signals(:,n_output*2+1:n_output*3,:);
tempo=linspace(0.0,T,size(x,3));

%% calcula a mdia e desvio
U=squeeze(x(:,1,:));
for zi=1:AN
    Um(zi,:)=U(zi,:)*ZW(zi);
end
xmean = sum(Um,1);
for zi=1:AN
    Uv(:,zi)=(U(zi,:)-xmean(1,:)).^2*ZW(zi);
end
Uvar=sum(Uv,2);xvar=Uvar';%sig=Uvar=variance.
%% plots no tempo
figure;  %Posio
lb=(2*xmean-sqrt(xvar*12))/2;%essa linha assume a hipotese de que a sada tem distribuio uniforme, mas isso no  necessariamente verdade.
ub=xmean+sqrt(xvar*12)/2;
plot (tempo,xmean,'k.-'); hold on;
plot (tempo,lb,'b.-');
plot (tempo,ub,'b.-');
title('Deslocamento - PCE'); legend('Mdia - PCE', 'Inferior - PCE', 'Superior - PCE');xlabel('Tempo (s)');ylabel('Posio (m)');
fprintf('      Total Number of function evaluations in PCE               =%5i\n',fcn_calls_PCE);

%% Trying to build the PDF (not working).
ns_k=5000;%PCE-MCS samples
imin=-1;imax=1;
nu=length(imin);             %Nmero de variveis incertas de entrada
samples=repmat(imin',1,ns_k) + repmat((imax-imin)',1,ns_k).*lhsdesign(ns_k,nu,'criterion','maximin')'; %Gerao de amostras uniformes dentro doslimites do intervalo
for i=1:level
    legendre_samples(i,:)=legendreP(i,samples);
end
MCS_PCE_surr=sum(legendre_samples);%sum the polynomials up to order 'level'
U_100=U(:,100);%pick the time 0.1s, QOI.
for i=1:size(U_100,1) 
    MCS_PCE_surr_n(i,:)=U_100(i).*MCS_PCE_surr;
end
MCS_PCE_surr_n2=sum(MCS_PCE_surr_n);
figure;
n_bins=20;
[N_pce,edges_pce] = histcounts(MCS_PCE_surr_n2(1,:), n_bins, 'Normalization','pdf');
hold on;title('Histogram PCE_{surr} + MCS (wrong) , t = 0.1s');
edges_pce_i = edges_pce(2:end) - (edges_pce(2)-edges_pce(1))/2;
plot(edges_pce_i, N_pce,'LineWidth',2);drawnow;%legend(meaning,'Location','southeast');
open 'MCS ans t=0.1s.fig'

%% salva as figuras e o Work Space
% save;
% FolderName = pwd;
% FigList = findobj(allchild(0), 'flat', 'Type', 'figure');
% for iFig = 1:length(FigList)
%   FigHandle = FigList(iFig);
%   FigName   = num2str(get(FigHandle, 'Number'));
%   set(0, 'CurrentFigure', FigHandle);
%   savefig(fullfile(FolderName, [FigName '.fig']));
% end